
# q-expansion of newform 6034.2.a.f, downloaded from the LMFDB on 14 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 6034
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [3], [2], [-1], [0], [1], [-2], [-7], [6], [5], [2], [8], [0], [-9], [6], [10], [4], [8], [1], [-15], [9], [1], [8], [-10], [16], [-15], [-4], [-12], [-5], [10], [12], [-10], [-12], [4], [-15], [-10], [-4], [-18], [-24], [0], [-12], [1], [3], [4], [-9], [-10], [-29], [12], [7], [14], [-18], [17], [1], [-20], [-17], [6], [-11], [-20], [2], [18], [4], [23], [7], [12], [6], [4], [28], [19], [28], [-10], [8], [-21], [-18], [12], [-30], [-8], [-7], [34], [-2], [-35], [20], [-10], [-1], [6], [8], [-38], [27], [2], [-24], [-26], [20], [-25], [11], [-36], [16], [9], [1], [12], [-1], [21], [22], [26], [21], [15], [3], [21], [44], [36], [35], [-33], [1], [9], [-28], [20], [-24], [-30], [19], [33], [18], [-4], [8], [-23], [40], [0], [-8], [-18], [25], [43], [-14], [-12], [1], [41], [46], [47], [-30], [26], [-34], [-38], [-12], [-16], [-11], [-18], [-26], [-50], [34], [50], [58], [3], [40], [-44], [-7], [36], [33], [-56], [12], [22], [-14], [5], [56], [-5], [57], [19], [23], [23], [21], [3], [-7], [-28], [27], [14], [38], [-50], [-6], [41], [-51], [33], [31], [-34], [-20], [4], [36], [-19], [-44], [-33], [-16], [-31], [-46], [13], [24], [49], [-32], [-12], [17], [6], [28], [-1], [10], [20], [-25], [25], [58], [27], [60], [-19], [56], [-14], [8], [-29], [66], [-52], [9], [-38], [-22], [-34], [-14], [37], [40], [-63], [-57], [-31], [7], [-48], [54], [-25], [8], [-58], [63], [-48], [-6], [41], [-67], [6], [55], [-12], [7], [-11], [4], [-29], [-45], [-15], [-40], [-33], [11], [-2], [15], [38], [6], [60], [-9], [36], [-62], [18], [18], [58], [-16], [-38], [54], [16], [6], [22], [-31], [-71], [-31], [-2], [45], [42], [12], [-49], [-21], [-13], [-10], [-50], [-62], [-24], [-34], [-5], [-76], [-26], [30], [-24], [57], [-53], [32], [23], [-8], [-56], [-74], [34], [-22], [-26], [-9], [77], [34], [25], [77], [-54], [-56], [3], [-34], [61], [-33], [63], [0], [-29], [69], [8], [-20], [-6], [-88], [-34], [46], [47], [-30], [-32], [44], [-37], [17], [-88], [70], [-50], [-44], [-72], [-66], [-43], [54], [16], [-4], [71], [75], [23], [-3], [53], [-40], [49], [-58], [80], [-74], [0], [-21], [-65], [5], [82], [30], [17], [-30], [-8], [-78], [-72], [44], [51], [-55], [68], [72], [-1], [-14], [68], [-29], [-55], [8], [-60], [-58], [90], [-13], [18], [30], [-11], [-23], [-8], [6], [-52], [20], [54], [-75], [57], [72], [9], [52], [-70], [-54], [-87], [-62], [44], [-88], [-14], [26], [-22], [41], [46], [-34], [-54], [-36], [31], [-48], [-19], [70], [-30], [-36], [15], [-62], [52], [76], [-72], [32], [-9], [38], [95], [4], [16], [-48], [54], [62], [92], [-77], [72], [70], [8], [72], [8], [46], [-78], [-54], [-27], [-69], [77], [63], [-23], [9], [30], [-18], [-30]]
